//=========================================================================
//
//	FILE:				CP_Document_Doc.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Document demo document class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 8 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:30 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// Document
#include "CP_Document_Doc.h"

	// CPLAT_Framework
#include "CP_Data_ResourceTypes.h"
#include "CP_Debugging.h"
#include "CP_Document.Tmpl.h"
#include "CP_DocumentWindow.h"
#include "CP_Environment.h"
#include "CP_File.h"
#include "CP_New.h"
#include "CP_MenuBarManager.h"
#include "CP_OSStatus.h"
#include "CP_PersistantWriter_Binary.h"
#include "CP_PersistantWritersManager.h"
#include "CP_PrintView.h"
#include "CP_SignalSlot.h"
#include "CP_Types.h"
#include "CP_Window.h"
#include "CP_WindowManager.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_Document_Doc
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inContainer			The scriptable object that serves as a container
*									for this document. In this case it will be the
*									application object.
*	@param		inDocType			The documents default file type.
*	
*	@exception	none
*/
CP_Document_Doc::CP_Document_Doc( CPLAT::CP_ScriptableObject *inContainer, CPLAT::FourCharCode inDocType )
	: inherited( inContainer, inDocType ),
		fTextView(NULL)
{
		// register our persistant writer objects for this document
	CPLAT::CP_WriterCreatorSignal			*binDataWriterSignal = CP_new CPLAT::CP_WriterCreatorSignal();
	binDataWriterSignal->connect( CPLAT::CP_SigSlot::slot(this, &CP_Document_Doc::CreateWriter) );
	CPLAT::CP_PersistantWritersManager::Instance()->RegisterWriter( CPLAT::kCP_BinaryDataType, binDataWriterSignal );
}

//=========================================================================
//	FUNCTION:	~CP_Document_Doc
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_Document_Doc::~CP_Document_Doc() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Creation ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DocumentCreator
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when some action has requested we create a new document object. This
*	is usually called from the CP_DocumentFactoryManager when it needs to create
*	a new document.
*
*	@param		inContainer			The scriptable object that serves as a container
*									for this document. In this case it will be the
*									application object.
*	@param		inDocumentClass		The documents default file type.
*	
*	@exception	none
*
*	@return		void
*/
CPLAT::CP_Document_Base *CP_Document_Doc::DocumentCreator( CPLAT::CP_ScriptableObject *inContainer,
	CPLAT::FourCharCode inDocumentClass )
{
	return( CP_new CP_Document_Doc( inContainer, inDocumentClass ) );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Main Window ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleCreateDocumentUI
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when the document wants to create the main user interface for the
*	document. In this demo we create a window, with a CP_TextView in it.
*
*	@param		inPostCreateFlags		Flags passed to this call to indicate
*										what type of action has caused this
*										to be called. The types of actions are:
*											- CP_E_Doc_Create_DoNothing
*											- CP_E_Doc_Create_CreateUI
*											- CP_E_Doc_Create_CreateUIOnNew
*											- CP_E_Doc_Create_CreateUIOnOpen
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::HandleCreateDocumentUI( SInt32 inPostCreateFlags )
{
	try
	{
		CPLAT::CP_DocumentWindow		*mainWindow = NULL;
		CPLAT::CP_Point					windowLocation( 50, 50 );
		CPLAT::CP_Size					windowSize( 400, 200 );
		CPLAT::CP_String				untitledString("Untitled");
		CPLAT::CP_String				windowTitle = GetDefaultFileName( untitledString );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		
		mainWindow = CP_new CPLAT::CP_DocumentWindow();
		mainWindow->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( mainWindow );
		
		CPLAT::CP_Point					paneLocation( 0, 0 );
		CPLAT::CP_Size					paneSize( 400, 200 );
		CPLAT::CP_String				paneTitle( "" );
		CPLAT::CP_DataValueSignal		*valueChangedSignal = NULL;

		fTextView = CP_new CPLAT::CP_TextView( mainWindow, paneLocation, paneSize, paneTitle, false );
		fTextView->SetFrameAdapter( CP_new CPLAT::CP_FrameAdapter_Sticky() );

			// connect to the value changed signal to update dirty flag
		valueChangedSignal = fTextView->FindSignal( kCP_Signal_ValueChanged );
		valueChangedSignal->connect( CPLAT::CP_SigSlot::slot( this, &CP_Document_Doc::TextFieldChanged ) );

			// associate our main window as the documents main window.
		SetMainWindow( mainWindow );
		
		CPLAT::CP_WindowManager::Instance()->Reposition( mainWindow, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );

		UInt16									itemCounter = 0;
		
		for( itemCounter = 0; itemCounter < 9; itemCounter++ )
		{
			CPLAT::CP_CommandHandlerSignal			*menuSignal = CP_new CPLAT::CP_CommandHandlerSignal();
			menuSignal->connect( CPLAT::CP_SigSlot::slot(this, &CP_Document_Doc::HandleSizeMenuCommand) );
			mainWindow->RegisterCommand( 1000 + itemCounter, menuSignal, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
		}

		for( itemCounter = 0; itemCounter < 4; itemCounter++ )
		{
			CPLAT::CP_CommandHandlerSignal			*menuSignal = CP_new CPLAT::CP_CommandHandlerSignal();
			menuSignal->connect( CPLAT::CP_SigSlot::slot(this, &CP_Document_Doc::HandleStyleMenuCommand) );
			mainWindow->RegisterCommand( 2000 + itemCounter, menuSignal, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
		}

		mainWindow->Show();
	}
	catch(...)
	{
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Opening ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleOpen
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when the user has requested we open a new document. In this demo
*	we will create a CP_File object to read in the data, and then set the contents
*	of the view to the data from the file. In this demo, no attempt is made
*	to verify that we are dealing with text data.
*
*	@param		inFileSpec				The CP_FileSystemSpec of the file that
*										is being opened.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::HandleOpen( const CPLAT::CP_FileSystemSpec& inFileSpec )
{
	CPLAT::CP_File			dataFile( inFileSpec );
	
	dataFile.Open();

	SInt64					dataLength = dataFile.Length();
	char					*buffer = (char *) CP_STD::malloc( (UInt32) (dataLength + 1) );

	dataFile.ReadData( buffer, (UInt32) dataLength );

	dataFile.Close();
	
	buffer[ dataLength ] = '\0';
	
	CPLAT::CP_String		text( buffer );
	
	if( fTextView != NULL )
		fTextView->SetText( text );
		
	CP_STD::free( buffer );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Persistant Writers ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CreateWriter
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the document is being saved. This will create a return a
*	CP_PersistantWriter object that is used to save the actual data.
*
*	@param		inReturnWriter			Location to return the newly created
*										persistant writer object to.
*	@param		inOSType				The OSType to determine the type of writer
*										to create. In this demo, we only create
*										a single type of writer. In other
*										cases, you may need to create different
*										persistant writers for different document
*										types.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::CreateWriter( CPLAT::CP_PersistantWriter **inReturnWriter,
	CPLAT::FourCharCode /*inOSType*/ )
{
	*inReturnWriter = CP_new CP_PersistantWriter_Binary( this );
}

//=========================================================================
//	FUNCTION:	GetFileTypeInfo
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when we need to get the file type information for our documents
*	file. In this demo, we supply the creator as the applications creator code,
*	and the files extensio as 'txt' for a text file.
*
*	@param		inFileSpec				Not used.
*	@param		inOutCreator			Location to return the creator code in.
*	@param		inOutType				Location to return the type code in.
*	@param		inOutFileExtension		Location to return the file extension for
*										this file.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::GetFileTypeInfo( const CPLAT::CP_FileSystemSpec& /*inFileSpec*/,
	CPLAT::FourCharCode *inOutCreator, CPLAT::FourCharCode *inOutType,
	CPLAT::CP_String *inOutFileExtension )
{
	if( inOutCreator != NULL )
		*inOutCreator = CPLAT::CP_Environment::GetCreator();
	*inOutType = CPLAT::kCP_BinaryDataType;
	*inOutFileExtension = "txt";
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Printing ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandlePrinting
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the user wants to print the contents of this document. In this
*	demo, we just ask the text view object to handle the OS native printing
*	provided by the text view.
*
*	@param		inOutStatus				Not used.
*	@param		inCommandNo				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::HandlePrintCommand( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& /*inCommandNo*/ )
{
	if( fTextView != NULL )
		fTextView->Print();
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Slots ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	TextFieldChanged
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the contents of the text view have changed. We use this
*	opportunity to mark the document as being dirty, and needing to be saved.
*
*	@param		inValue					Not used.
*	@param		inOutStatus				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::TextFieldChanged( CPLAT::CP_DataValueHolder& /*inValue*/,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	SetDirty( true );
}

//=========================================================================
//	FUNCTION:	HandleSizeMenuCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Call when the user selects a size from the size menu
*
*	@return		inOutStatus			Not used.
*	@return		inCommandNo			The command number associated with the selected item.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::HandleSizeMenuCommand( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& inCommandNo )
{
	SInt16			fontSize = 12;
	
	switch( inCommandNo )
	{
		case 1001:
			fontSize = 9;
			break;
		case 1002:
			fontSize = 10;
			break;
		case 1003:
			fontSize = 11;
			break;
		case 1004:
			fontSize = 12;
			break;
		case 1005:
			fontSize = 14;
			break;
		case 1006:
			fontSize = 16;
			break;
		case 1007:
			fontSize = 18;
			break;
		case 1008:
			fontSize = 24;
			break;
	}
	
	if( fTextView != NULL )
		fTextView->SetFontSize( fontSize );
}

//=========================================================================
//	FUNCTION:	HandleStyleMenuCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Call when the user selects a style from the size menu
*
*	@return		inOutStatus			Not used.
*	@return		inCommandNo			The command number associated with the selected item.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::HandleStyleMenuCommand( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& inCommandNo )
{
	SInt16			fontStyle = CPLAT::CP_E_FontStyle_Normal;
	
	switch( inCommandNo )
	{
		case 2001:
			fontStyle = CPLAT::CP_E_FontStyle_Normal;
			break;
		case 2002:
			fontStyle = CPLAT::CP_E_FontStyle_Italic;
			break;
		case 2003:
			fontStyle = CPLAT::CP_E_FontStyle_Bold;
			break;
	}
	
	if( fTextView != NULL )
		fTextView->SetFontStyle( fontStyle );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Write Data ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	WritePersistantData
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called from the persistant writer we created above to actualy write out
*	the documents data. In this demo we write out the contents of the text view
*	to the passed in file. No formatting or any other special information is written,
*	just the plain text.
*
*	@param		inFile				The CP_File object that we are writing to. The
*									file has already been opened for writing into.
*	@param		inWriter			The CP_PersistantWriter object that is handling
*									this operation.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Doc::WritePersistantData( CPLAT::CP_File& inFile, CPLAT::CP_PersistantWriter */*inWriter*/ )
{
	ASSERT( fTextView != NULL );
	
	if( fTextView != NULL )
	{
		UInt32					bytesWritten = 0;
		CPLAT::CP_String		textViewContents = fTextView->GetText();
		char					*buffer = const_cast<char *>(textViewContents.c_str());
		UInt32					bufferSize = textViewContents.length();
		
		bytesWritten = inFile.WriteData( buffer, bufferSize );

			// since data is written, text field is no longer dirty
		SetDirty( false );
	}
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
