//=========================================================================
//
//	FILE:				CP_Document_Doc.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Document demo document class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 8 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:30 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Document_Doc
#define _H_CP_Document_Doc
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Document.h"
#include "CP_File.h"
#include "CP_FileSystemSpec.h"
#include "CP_OSTypes.h"
#include "CP_PrintPolicy_Print.h"
#include "CP_ScriptableObject.h"
#include "CP_TextView.h"
#include "CP_Types.h"
#include "CP_UndoPolicy_None.h"

/*!
*	@class		CP_Document_Doc
*	@brief		CP_Document class to handle the data from the main window and reading / writing contents.
*/
class CP_EXPORT CP_Document_Doc : public CPLAT::CP_Document<CPLAT::CP_UndoPolicy_None, CPLAT::CP_PrintPolicy_Print>,
	CPLAT::CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Document_Doc(
												CPLAT::CP_ScriptableObject *inContainer,
												CPLAT::FourCharCode inDocType );
	virtual								~CP_Document_Doc() throw();
	//@}
			
	//! @name Creator
	//@{
	static	CPLAT::CP_Document_Base	 	*DocumentCreator(
												CPLAT::CP_ScriptableObject *inContainer,
												CPLAT::FourCharCode inDocumentClass );
	//@}

	//! @name Document UI
	//@{
	virtual	void						HandleCreateDocumentUI( SInt32 inPostCreateFlags = CP_E_Doc_Create_DoNothing );
	//@}

	//! @name File
	//@{
	virtual	void						HandleOpen( const CPLAT::CP_FileSystemSpec& inFileSpec );

	virtual	void						WritePersistantData(
												CPLAT::CP_File& inFile,
												CPLAT::CP_PersistantWriter *inWriter );
	//@}

protected:
	//! @name Persistance
	//@{
			void						CreateWriter(
												CPLAT::CP_PersistantWriter **inReturnWriter,
												CPLAT::FourCharCode inOSType );

	virtual	void						GetFileTypeInfo(
												const CPLAT::CP_FileSystemSpec& inFileSpec,
												CPLAT::FourCharCode *inOutCreator,
												CPLAT::FourCharCode *inOutType,
												CPLAT::CP_String *inOutFileExtension );
	//@}

	//! @name Printing
	//@{
	virtual	void						HandlePrintCommand(
												CPLAT::OSStatus& inOutStatus,
												CPLAT::CP_CommandNo& inCommandNo );
	//@}

private:
	//! @name Slots
	//@{
			void 						TextFieldChanged(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );

			void 						HandleSizeMenuCommand(
												CPLAT::OSStatus& inOutStatus,
												CPLAT::CP_CommandNo& inCommandNo );
			void 						HandleStyleMenuCommand(
												CPLAT::OSStatus& inOutStatus,
												CPLAT::CP_CommandNo& inCommandNo );
	//@}

	// class data
protected:
	CPLAT::CP_TextView					*fTextView;
	
private:
	typedef CPLAT::CP_Document<CPLAT::CP_UndoPolicy_None, CPLAT::CP_PrintPolicy_Print>	inherited;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif	// _H_CP_Document_Doc

