//=========================================================================
//
//	FILE:				Printing_View.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			View class to demonstrate Printing.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 5 $
//						Latest Checkout:  $Date: 2005-08-30 20:33:00 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// Printing demo
#include "Printing_View.h"

	// CPLAT_Framework
#include "CP_Color.h"
#include "CP_Graphics.h"
#include "CP_Numeric.h"
#include "CP_RGBColor.h"
#include "CP_Utility.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	Printing_View
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The subview handler that contains this pane.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the pane.
*	
*	@exception	none
*/
Printing_View::Printing_View( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize )
		: CPLAT::CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
}

//=========================================================================
//	FUNCTION:	~Printing_View
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
Printing_View::~Printing_View() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	CP_View_Imp method called from the user panes callback.
*
*	@param	inCanvas			CP_Canvas object to draw into.
*	@param	inUpdateArea		Update area to draw into.
*
*	@return	void
*/
void Printing_View::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Rect				bounds = GetLocalBounds();
	CPLAT::CP_Rect				portBounds = GetPortBounds();
	CPLAT::CP_Graphics			graphics( inCanvas );
	CPLAT::CP_FontSpec			fontSpec;
	CPLAT::CP_String			text;
	
	bounds.Inset( 2, 2 );
	graphics.StrokeRect( bounds );
	
	text = "H: ";
	text += CPLAT::CP_Utility::CP_SInt32ToString( CPLAT::numeric_cast<SInt32>(portBounds.Left()) );
	text += " V: ";
	text = CPLAT::CP_Utility::CP_SInt32ToString( CPLAT::numeric_cast<SInt32>(portBounds.Top()) );
	fontSpec.SetSize( 10 );
	graphics.DrawText( fontSpec, text, bounds, CPLAT::CP_E_TextAlign_HorCenter, CPLAT::CP_E_TextAlign_VerMiddle );
}

#ifdef TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
