//=========================================================================
//
//	FILE:				CP_ByteSwappers.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Various routines for swapping bytes between little and
//						big edian systems.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 115 $
//						Latest Checkout:  $Date: 2005-03-24 16:55:37 -0800 (Thu, 24 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_ByteSwappers
#define _H_CP_ByteSwappers
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ByteSwappers
*	@brief		The CP_ByteSwappers are used to byte swap SInt16, UInt16, SInt32 and UInt32.
*/

//=========================================================================
//	FUNCTION:	BYTE_SWAP_WORD
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
 *	Swaps the bytes in the passed in SInt16 and returns the result.
 *	
 *	@param		inShort				The SInt16 to be byte swapped.
 *	
 *	@exception	none
 *
 *	@return		SInt16				The SInt16 with its bytes swapped.
 */
inline SInt16		BYTE_SWAP_WORD( register SInt16 inShort )		{ return (SInt16)(((inShort>>8)&0x00ff) | ((inShort<<8)&0xff00)); }			

//=========================================================================
//	FUNCTION:	BYTE_SWAP_UWORD
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
 *	Swaps the bytes in the passed in UInt16 and returns the result.
 *	
 *	@param		inUShort			The UInt16 to be byte swapped.
 *	
 *	@exception	none
 *
 *	@return		UInt16				The UInt16 with its bytes swapped.
 */
inline UInt16		BYTE_SWAP_UWORD( register UInt16 inUShort )		{ return (UInt16)(((inUShort>>8)&0x00ff) | ((inUShort<<8)&0xff00)); }

//=========================================================================
//	FUNCTION:	BYTE_SWAP_LONG
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
 *	Swaps the bytes in the passed in SInt32 and returns the result.
 *	
 *	@param		inLong				The SInt32 to be byte swapped.
 *	
 *	@exception	none
 *
 *	@return		SInt32				The SInt32 with its bytes swapped.
 */
inline SInt32		BYTE_SWAP_LONG( register SInt32 inLong )		{ return (SInt32)(((inLong<<24)&0xff000000) | ((inLong<<8)&0x00ff0000) | ((inLong>>8)&0x0000ff00) | ((inLong>>24)&0x000000ff)); }

//=========================================================================
//	FUNCTION:	BYTE_SWAP_ULONG
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
 *	Swaps the bytes in the passed in UInt32 and returns the result.
 *	
 *	@param		inULong				The UInt32 to be byte swapped.
 *	
 *	@exception	none
 *
 *	@return		UInt32				The UInt32 with its bytes swapped.
 */
inline UInt32		BYTE_SWAP_ULONG( register UInt32 inULong )		{ return ((inULong<<24)&0xff000000) | ((inULong<<8)&0x00ff0000) | ((inULong>>8)&0x0000ff00) | ((inULong>>24)&0x000000ff); }

//=========================================================================
//	FUNCTION:	CP_BYTE_SWAP
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Swaps the bytes in the passed in SInt16 and returns the result. This just calls
*	BYTE_SWAP_WORD to do the actual conversion.
*	
*	@param		inShort				The SInt16 to be byte swapped.
*	
*	@exception	none
*
*	@return		SInt16				The SInt16 with its bytes swapped.
*/
inline SInt16 CP_BYTE_SWAP( SInt16 inShort )						{ return( BYTE_SWAP_WORD( inShort ) ); }

//=========================================================================
//	FUNCTION:	CP_BYTE_SWAP
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Swaps the bytes in the passed in UInt16 and returns the result. This just calls
*	BYTE_SWAP_UWORD to do the actual conversion.
*	
*	@param		inUShort			The UInt16 to be byte swapped.
*	
*	@exception	none
*
*	@return		UInt16				The UInt16 with its bytes swapped.
*/
inline UInt16 CP_BYTE_SWAP( UInt16 inUShort )						{ return( BYTE_SWAP_UWORD( inUShort ) ); }

//=========================================================================
//	FUNCTION:	CP_BYTE_SWAP
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Swaps the bytes in the passed in SInt32 and returns the result. This just calls
*	BYTE_SWAP_UWORD to do the actual conversion.
*	
*	@param		inLong				The SInt32 to be byte swapped.
*	
*	@exception	none
*
*	@return		SInt32				The SInt32 with its bytes swapped.
*/
inline SInt32 CP_BYTE_SWAP( SInt32 inLong )						{ return( BYTE_SWAP_LONG( inLong ) ); }

//=========================================================================
//	FUNCTION:	CP_BYTE_SWAP
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Swaps the bytes in the passed in UInt32 and returns the result. This just calls
*	BYTE_SWAP_UWORD to do the actual conversion.
*	
*	@param		inULong				The UInt32 to be byte swapped.
*	
*	@exception	none
*
*	@return		UInt32				The UInt32 with its bytes swapped.
*/
inline UInt32 CP_BYTE_SWAP( UInt32 inUInt32 )						{ return( BYTE_SWAP_ULONG( inUInt32 ) ); }

//=========================================================================
//	FUNCTION:	CP_BYTE_SWAP
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Swaps the bytes in the passed in float
*	
*	@param		inFloat				The float to be byte swapped.
*	
*	@exception	none
*
*	@return		void
*/
inline void	CP_BYTE_SWAP( float inFloat )							{ SInt32 *longPtr = (SInt32 *) &inFloat;  *longPtr = BYTE_SWAP_LONG( (SInt32&) inFloat); }

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ByteSwappers



