//=========================================================================
//
//	FILE:				CP_Dialog.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for handling dialogs in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 198 $
//						Latest Checkout:  $Date: 2005-06-01 23:15:21 -0700 (Wed, 01 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Dialog
#define _H_CP_Dialog
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DialogModality.h"
#include "CP_Object.h"
#include "CP_Point.h"
#include "CP_SignalSlot.h"
#include "CP_Size.h"
#include "CP_String.h"
#include "CP_Window.h"
#include "CP_WindowTypes.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Dialog
*	@brief		Base class for handling dialogs in CPLAT.
*/
class CP_EXPORT CP_Dialog : public CP_Object, public CP_SigSlot::CP_SlotObject
{
public:
	enum
	{
		kCP_DialogModality_Modal,
		kCP_DialogModality_Modeless
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Dialog();
	//! Destructor.
	virtual								~CP_Dialog() throw();
	//@}
			
	//! @name Initialization
	//@{
			void						Create(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inWindowTitle,
												const CP_WindowTypeBase& inWindowType );
			void						CreateButtons(
												const CP_String& inDefaultButtonText,
												const CP_String& inCancelButtonText );
	//@}

	//! @name Events
	//@{
			void						Run( SInt16 inModal = kCP_DialogModality_Modal );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the CP_Window used to display the dialogs views.
										*	
										*	@exception	none
										*	
										*	@return		CP_Window		The dialogs window.
										*/
			CP_Window					*GetWindow()												{ return( fDialogWindow ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the dialogs modality.
										*
										*	@param		inModality		The CP_DialogModality to control the
										*								modality of this dialog.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetModality( CP_DialogModality *inModality )				{ fDialogModality = inModality; }
			void						SetModality(
												WindowModality inWindowModality,
												CP_Window *inParentWindow = NULL );
	//@}

protected:
	//! @name Slots
	//@{
			void						DismissButtonClicked(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
			void						CancelButtonClicked(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	//@}

	// class data
protected:
	CP_Window							*fDialogWindow;			/*! CP_Window used to display the dialog. */
	
	CP_DialogModality					*fDialogModality;		/*! Modality of the dialog. */
	
	CP_DataValueSignal					fDismissDialogSignal;	/*! Signal sent when the dismiss button is clicked. */
	CP_DataValueSignal					fCancelDialogSignal;	/*! Signal sent when the cancel button is clicked. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Dialog

