//=========================================================================
//
//	FILE:				CP_DocNodeDecoders.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A set of classes that can decode the data presented
//						an XMLDocuments CP_XMLDocNodes to CPLAT data types.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 142 $
//						Latest Checkout:  $Date: 2005-04-10 21:14:06 -0700 (Sun, 10 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_DocNodeDecoders
#define _H_CP_DocNodeDecoders
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_Menu.h"
#include "CP_NoCopy.h"
#include "CP_TabView.h"
#include "CP_XMLDocument.h"

CPLAT_Begin_Namespace_CPLAT

/*! Callback typedef to Decode a CP_M_Persist object.. */
typedef CP_DataValueHolder		(*DecodeCallback)( CP_XMLDocNode *inXMLDocNode );

typedef std::map<CP_String, DecodeCallback, std::less<CP_String> >				CP_Decoders_Map;
typedef std::map<CP_String, DecodeCallback, std::less<CP_String> >::iterator	CP_Decoders_Iterator;

typedef std::pair<const CP_String, DecodeCallback>								CP_Decoders_Entry;

/*!
*	@class	CP_DocNodeDecordersManager
*	@brief	Singleton class to manage CP_M_Persist object.
*/
class CP_EXPORT CP_DocNodeDecordersManager : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DocNodeDecordersManager();
	//! Destructor.
	virtual								~CP_DocNodeDecordersManager();
	//@}

	//! @name Instance
	//@{
	static	CP_DocNodeDecordersManager	*Instance();
	//@}

	//! @name Decoders
	//@{
			void						RegisterDecoder(
												const CP_String& inClassName,
												DecodeCallback inDecoderFunction );

			DecodeCallback				FindDecoder( const CP_String& inClassName );
	//@}

protected:

private:

	// class data
protected:	
	static CP_DocNodeDecordersManager	*fsInstance;	/*! Single instance of this class. */

	CP_Decoders_Map						*fDecoders;		/*! List of available decoders. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#define CP_RegisterXMLDecoder( inClassName, inDecoderFunction )		CP_DocNodeDecordersManager::Instance()->RegisterDecoder( inClassName, inDecoderFunction )

	// Decoders
CP_DataValueHolder		XMLDecoder_CPRect( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_CPPoint( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_CPRGBColor( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_CPSize( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_CPString( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_Bool( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_SInt16( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_UInt16( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_SInt32( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_UInt32( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder 		XMLDecoder_OSType( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder		XMLDecoder_Float( CP_XMLDocNode *inXMLDocNode );
CP_DataValueHolder 		XMLDecoder_TabItems( CP_XMLDocNode *inXMLDocNode );
CP_TabInfo 				XMLDecoder_TabInfo( CP_XMLDocNode *inTabInfoNode );
CP_DataValueHolder		XMLDecoder_CPMenu( CP_XMLDocNode *inXMLDocNode );
void					XMLDecoder_CPMenuItems_private(
								CP_XMLDocNode *inXMLDocNode,
								CP_Menu *inMenu );

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DocNodeDecoders

