/*
=========================================================================

	FILE:		CP_LimitSingleInstance.h
	
				Copyright  2001. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	A class for Windows applications to limit to running a single
				instance of the application.
	
	HISTORY:	1st Version - November 2001.
	
	NOTES:		This code is based on the following Microsoft Knowledge Base
				article: Q243953 - Howto: Limit 32-bit Applications to One Instance.

	VERSION:	Current Revision: $Revision: 2 $
				Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $

=========================================================================
*/
#ifndef _H_CP_LimitSingleInstance
#define _H_CP_LimitSingleInstance
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CP_Framework
#include "CP_String.h"
#include "CP_Types.h"

#if TARGET_OS_WIN32
	#include <windows.h> 
#endif

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_LimitSingleInstance
{
public:
									// Construction / Destruction
								CP_LimitSingleInstance( const CP_String& inStrMutexName );
								~CP_LimitSingleInstance() ;

			bool				IsAnotherInstanceRunning();

protected:
	#if TARGET_OS_WIN32
		DWORD			fLastError;
		HANDLE			fMutex;
	#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_LimitSingleInstance
