//=========================================================================
//
//	FILE:				CP_Message.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A message object that is send from CP_M_Broadcasters to
//						CP_M_Listener objects in CP Framework.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Message
#define _H_CP_Message
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// Forward declarations
class CP_EXPORT CP_Message
{
public:
									// Construction / destruction
										CP_Message();
										CP_Message(
												void *inSendingObject,
												SInt32 inMsgType,
												SInt32 inMessage,
												SInt32 inData = 0 );
	virtual								~CP_Message();
	
			void						Clear();

										/*! Return the SInt32 of this message. */
			SInt32						GetMessage() const													{ return( fMessage ); }
										/*! Return the sending object. */
			void 						*GetSender() const													{ return( fSendingObject ); }
										/*! Return the type of message contained in this message.*/
			SInt32						GetType() const														{ return( fMessageType ); }
										/*! Return any user supplied data. */
			SInt32						GetData() const														{ return( fData ); }
			
										/*! Set the SInt32 for this message. */
			void						SetMessage( const SInt32 inMessage )								{ fMessage = inMessage; }
										/*! Sets the sending object of this message. */
			void						SetSender( void *inSender = NULL )									{ fSendingObject = inSender; }
										/*! Sets the type of message this is. */
			void						SetType( const SInt32 inType )										{ fMessageType = inType; }
										/*! Sets any user supplied data for this message. */
			void						SetData( SInt32 inData )											{ fData = inData; }

	// class data
protected:
										/*! The message. */
	SInt32								fMessage;			// message being sent
										/*! The type of message being sent. */
	SInt32								fMessageType;		// type of message
										/*! The sending object. */
	void								*fSendingObject;	// the object that sent this message
										/*! Data attached to this message. */
	SInt32								fData;				// misc data attached to this message
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Message


