//=========================================================================
//
//	FILE:				CP_OSStatus.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Template based class to represent an error code in
//						CPLAT. This is represented as lhs SInt32 type and the
//						equivalent of lhs Mac OSStatus error code.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_OSStatus
#define _H_CP_OSStatus
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Errors.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

//!  OSStatus
/*!
  Template based class to represent an error code in CPLAT. This is represented as
  an SInt32 type and the equivalent of the Mac OSStatus error code. 
*/
class OSStatus
{
public:
	typedef SInt32 ErrorStatus;
     
											// Construction / destruction
										OSStatus()
											: fStatus( CP_noErr )								{}
										OSStatus( SInt32 inStatus )
											: fStatus( inStatus )							{}
										OSStatus( ::OSErr inStatus )
											: fStatus( inStatus )							{}
										OSStatus( signed int inStatus )
											: fStatus( inStatus )							{}
     
	static	OSStatus					Make( SInt32 inStatus )								{ return( OSStatus( inStatus ) ); }
			SInt32						Get() const                         				{ return( fStatus ); }
			operator SInt32() const                           								{ return( fStatus ); }

	// class data
private:
	SInt32								fStatus;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_OSStatus

