//=========================================================================
//
//	FILE:				CP_OffscreenCanvas.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A CP_Canvas object for management of drawing offscreen.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 258 $
//						Latest Checkout:  $Date: 2005-08-27 13:57:43 -0700 (Sat, 27 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_OffscreenCanvas
#define _H_CP_OffscreenCanvas
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Bitmap.h"
#include "CP_Canvas.h"
#include "CP_Enums.h"
#include "CP_Palette.h"
#include "CP_Rect.h"
#include "CP_Region.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_OffscreenCanvas
*	@brief		A CP_Canvas object for management of drawing offscreen.
*/
class CP_EXPORT CP_OffscreenCanvas : public CP_Canvas
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_OffscreenCanvas(
												const CP_Rect& inBounds,
												CP_Canvas& inDestCanvas );
	//! Destructor.
	virtual								~CP_OffscreenCanvas() throw();
	//@}
			
	//! @name Drawing
	//@{
			bool						BeginDraw();
			void						EndDraw();
			
				// preferred versions
			void						CopyImage(
												CP_Canvas& inDestCanvas,
												const CP_Rect& inDestBounds );

				// older - depreciated in 10.4 and above
			void						CopyImage(
												OSGraphicsPort inDestPort,
												const CP_Rect& inDestBounds,
												SInt32 inTransferMode = CP_E_TransferMode_SrcCopy );
			void						CopyImage(
												OSGraphicsPort inDestPort,
												const CP_Rect& inDestBounds,
												const CP_Region& inMaskRegion,
												SInt32 inTransferMode = CP_E_TransferMode_SrcCopy );
			void						CopyImage(
												OSGraphicsPort inDestPort,
												const CP_Rect& inSrcBounds,
												const CP_Rect& inDestBounds,
												const CP_Region& inMaskRegion,
												SInt32 inTransferMode = CP_E_TransferMode_SrcCopy );
			void						CopyImageWithBlend(
												OSGraphicsPort inDestPort,
												const CP_Rect& inSrcArea,
												const CP_Rect& inDestArea,
												const CP_Region& inMaskRegion,
												double inAlphaValue );
			void						CopyImageWithBlend(
												CP_Canvas& inDestCanvas,
												const CP_Rect& inSrcArea,
												const CP_Rect& inDestArea,
												const CP_Region& inMaskRegion,
												double inAlphaValue );
	//@}

	//! @name Getters
	//@{
			CP_Bitmap					*GetBitmap();
			CP_Rect						GetBounds() const							{ return( fBounds ); }
	//@}

	//! @name Getters
	//@{
			void						SetTransferMode( SInt32 inTransferMode )			{ fTransferMode = inTransferMode; }
	//@}

protected:

	// class data
protected:
	CP_Rect								fBounds;
	
	SInt32								fTransferMode;

	#if TARGET_OS_MAC
		GWorldPtr						fGWorld;

		CGrafPtr						fSavePort;
		GDHandle						fSaveDevice;

		CP_RGBColor						fCopyBitsForeColor;
		CP_RGBColor						fCopyBitsBackColor;
	
		#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
			CGLayerRef					fCGLayerRef;
		#endif // TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
	#endif
	
	#if TARGET_OS_WIN32
		CP_Bitmap						*fCanvasBitmap;

		OSGraphicsPort					fOffscreenPort;
		HBITMAP							fOffscreenBitmap;
		HBITMAP							fPreviousBitmap;
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_OffscreenCanvas(
												const CP_Rect& inBounds,
												SInt16 inPixelDepth,
												CP_Palette *inPalette = NULL,
												GDHandle inGDevice = NULL,
												GWorldFlags inFlags = 0 );

	//@}

	#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
	//! @name CGLayers
	//@{
		CGLayerRef						GetCGLayer()										{ return( fCGLayerRef ); }
	//@}

	//! @name ContextRef
	//@{
	virtual	bool						HasValidContextRef();
	virtual	CGContextRef				GetContextRef( bool inSyncOriginWithPort = true );
	//@}
	#endif // TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)

	//! @name GWorld
	//@{
		GWorldPtr						GetGWorld()											{ return( fGWorld ); }
	//@}

	//! @name Setters
	//@{
			void						SetCopyBitsColors(
												const CP_RGBColor& inForeColor,
												const CP_RGBColor& inBackColor );
	//@}

protected:
	//! @name GWorld
	//@{
			void						CreateGWorld(
												SInt16 inPixelDepth,
												const CP_Rect& inBounds,
												CTabHandle inColorTable,
												GDHandle inGDevice,
												GWorldFlags inFlags );
	//@}


	//! @name Cleanup
	//@{
			void						Cleanup();
	//@}

private:
	//! @name Misc
	//@{
			void						PrepCopyBits();
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Cleanup
	//@{
			void						Cleanup();
	//@}
#endif

#if TARGET_OS_LINUX
protected:
	//! @name Cleanup
	//@{
			void						Cleanup();
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_OffscreenCanvas

