//=========================================================================
//
//	FILE:				CP_PersistantWritersManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Singleton class to manage registration and creation
//						of CP_PersistantWriter objects to write data out to
//						peristant data storage.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 160 $
//						Latest Checkout:  $Date: 2005-04-27 21:21:26 -0700 (Wed, 27 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_PersistantWritersManager
#define _H_CP_PersistantWritersManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_OSTypes.h"
#include "CP_Persistant.h"
#include "CP_PersistantWriter.h"

	// Standard Library
#include <vector>


#include "CP_SignalSlot.h"

CPLAT_Begin_Namespace_CPLAT

typedef CP_SigSlot::CP_Signal2<CP_PersistantWriter **, FourCharCode>					CP_WriterCreatorSignal;

class CP_PersistantWriterEntry
{
public:
											// Construction / destruction
										CP_PersistantWriterEntry();
										CP_PersistantWriterEntry( const CP_PersistantWriterEntry& inCommand );

											// Setters
		FourCharCode					GetType()																{ return( fType ); }
		CP_WriterCreatorSignal			*GetCreatorSignal()														{ return( fWriterCreatorSignal ); }

											// operators
		CP_PersistantWriterEntry&		operator =( const CP_PersistantWriterEntry& inEntry );
									
											// Setters
		void							SetType( FourCharCode inType )											{ fType = inType; }
		void							SetCreator( CP_WriterCreatorSignal *inCreator )							{ fWriterCreatorSignal = inCreator; }

	// class data
private:
	FourCharCode						fType;
	CP_WriterCreatorSignal				*fWriterCreatorSignal;
};

typedef std::vector<CP_PersistantWriterEntry, std::allocator<CP_PersistantWriterEntry> >						CP_PersistantWriterCreator_Array;
typedef std::vector<CP_PersistantWriterEntry, std::allocator<CP_PersistantWriterEntry> >::iterator				CP_PersistantWriterCreator_Iterator;
typedef std::vector<CP_PersistantWriterEntry, std::allocator<CP_PersistantWriterEntry> >::reverse_iterator		CP_PersistantWriterCreator_ReverseIterator;

/*!
*	@class	CP_PersistantWritersManager
*	@brief	Singleton class to create CP_PersistantWriter objects.
*	
*	Singleton class to manage registration and creation of CP_PersistantWriter
*	objects to write data out to peristant data storage.
*/
class CP_EXPORT CP_PersistantWritersManager : public CP_NoCopy
{
public:
										// Construction / destruction
										CP_PersistantWritersManager();
	virtual								~CP_PersistantWritersManager();
	
	static	CP_PersistantWritersManager	*Instance();

			CP_PersistantWriter			*FindWriter( FourCharCode inType );
			void						RegisterWriter(
												FourCharCode inType,
												CP_WriterCreatorSignal *inCreator );
			bool						IsRegisteredType( FourCharCode inType );
			void						ReplaceCreator(
												FourCharCode inType,
												const CP_PersistantWriterEntry& inCreatorEntry );

protected:

	// class data
protected:
	static CP_PersistantWritersManager	*fsInstance;

	CP_PersistantWriterCreator_Array	*fCreators;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PersistantWritersManager

