//=========================================================================
//
//	FILE:				CP_QuickTime_Image.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle management of QuickTime image functions.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 269 $
//						Latest Checkout:  $Date: 2005-09-05 19:20:19 -0700 (Mon, 05 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_QuickTime_Image
#define _H_CP_QuickTime_Image
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_OffscreenCanvas.h"
#include "CP_Rect.h"
#include "CP_View.h"

#if TARGET_OS_MAC
	#if TARGET_API_MAC_CARBON && __CF_USE_FRAMEWORK_INCLUDES__
		#include <QuickTime/ImageCompression.h>
		#include <QuickTime/QuickTimeComponents.h>
	#else
		#include <ImageCompression.h>
		#include <QuickTimeComponents.h>
	#endif
#endif

#if TARGET_OS_WIN32
	#if CP_Optional_QuickTime_Windows
		#include <ImageCompression.h>
		#include <QuickDraw.h>
		#include <QuickTimeComponents.h>
	#endif
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class		CP_QuickTime_Image
*	\brief		A class to handle management of QuickTime image functions.
*/
class CP_EXPORT CP_QuickTime_Image
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_QuickTime_Image();
	//! Destructor.
	virtual								~CP_QuickTime_Image() throw();
	//@}
			
	//! @name Drawing
	//@{
			void						DrawImage(
												CP_OffscreenCanvas& inDestCanvas,
												const CP_Rect& inDestBounds );
			void						DrawImage(
												CP_Canvas& inDestCanvas,
												const CP_Rect& inDestBounds );
	//@}


	//! @name Export
	//@{
			void 						ExportImage(
												const CP_FileSystemSpec& inDestSpec,
												OSType inImageType = 'JPEG'/*kQTFileTypeJPEG*/ );
			void 						ExportImage(
												const CP_FileSystemSpec& inDestSpec,
												PicHandle inImageData );
			void 						ExportImage(
												const CP_FileSystemSpec& inDestSpec,
												CP_OffscreenCanvas& inCanvas );
	//@}

	//! @name Getters
	//@{
			CP_Rect						GetImageBounds();
	//@}

	//! @name Import
	//@{
			void						ImportImage( const CP_FileSystemSpec& inFileSpec );
	//@}

protected:

	// class data
protected:

	#if TARGET_OS_MAC || CP_Optional_QuickTime_Windows
		GraphicsImportComponent			fGraphicsImportComponent;
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
			GraphicsImportComponent		GetGraphicsImportComponent()				{ return( fGraphicsImportComponent ); }
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_QuickTime_Image
