//=========================================================================
//
//	FILE:				CP_RGBColor.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle management of a color in RGB format,
//						and provide conversion to and from the underlying OSs
//						native format of a color representation.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_RGBColor
#define _H_CP_RGBColor
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

#if TARGET_OS_LINUX
	#include <gtk/gtk.h>
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_RGBColor
*	@brief		A class to handle management of a color in RGB format, and provide conversion
*				to and from the underlying OSs native format of a color representation.
*/
class CP_EXPORT CP_RGBColor 
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_RGBColor();
										CP_RGBColor(
											UInt16 inRed,
											UInt16 inGreen,
											UInt16 inBlue );
										CP_RGBColor( const CP_RGBColor& inColor );
	//! Destructor.
	virtual								~CP_RGBColor();
	//@}

	//! @name Getters
	//@{
			UInt16						red() const;
			UInt16						green() const;
			UInt16						blue() const;
	//@}
			
	//! @name operators
	//@{
			bool						operator == ( const CP_RGBColor& inColor ) const;
			bool						operator != ( const CP_RGBColor& inColor ) const;

			CP_RGBColor&				operator = ( const CP_RGBColor& inColor );
	//@}

	//! @name Setters
	//@{
			void						Set(
												UInt16 inRed,
												UInt16 inGreen,
												UInt16 inBlue );
		
										/*!
										*	Sets the color from the passed in CP_RGBColor object.
										*
										*	@param		inColor		A CP_RGBColor object to set the color from.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						Set( const CP_RGBColor& inColor )					{ operator = ( inColor ); }

			void						Lighten();
			void						Darken();
	//@}

	// class data
protected:
									
	UInt16								fRed;		/*! The red color component. */
	UInt16								fGreen;		/*! The green color component. */
	UInt16								fBlue;		/*! The blue color component. */

	#if TARGET_OS_LINUX
		GdkColor						fGDKColor;	/*! The Linux reprentation of a color. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_RGBColor( RGBColor inColor );
	//@}

	//! @name Converters
	//@{
										operator RGBColor *();
										operator const RGBColor *() const;
							
										operator RGBColor();
										operator const RGBColor() const;
	//@}

	//! @name Getters
	//@{
			RGBColor					GetRGBColor();
			RGBColor					GetRGBColor() const;
	//@}

	//! @name Setters
	//@{
			void						Set( const RGBColor inColor );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_RGBColor( COLORREF inColor );
	//@}

	//! @name Converters
	//@{
			COLORREF					GetCOLORREF() const;

										/*!
										*	COLORREF operator to return the color as a Windows COLORREF.
										*	
										*	@exception	none
										*
										*	@return		COLORREF		Windows COLORREF of this color.
										*/
										operator COLORREF () const							{ return GetCOLORREF(); }
	//@}

	//! @name Setters
	//@{
			void						Set( COLORREF inColor );
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Converters
	//@{
			GdkColor					*GetGDKColor();
	//@}

	//! @name Setters
	//@{
			void						Set( GdkColor inColor );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_RGBCOLOR

