//=========================================================================
//
//	FILE:				CP_AE_List.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Wrapper class for dealing with Apple Event descriptors that
//						represent a list.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 188 $
//						Latest Checkout:  $Date: 2005-05-22 13:21:24 -0700 (Sun, 22 May 2005) $
//
//=========================================================================
#ifndef _H_CP_AE_List
#define _H_CP_AE_List
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

#include "CP_AE_Descriptor.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_AE_List
*	@brief		The CP_AE_List class is a MacOS only class that provides for Apple Event descriptor lists.
*
*	@ingroup	AppleEvents
*/
class CP_EXPORT CP_AE_List : public CP_AE_Descriptor
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AE_List();
	//! Destructor.
	virtual								~CP_AE_List();
	//@}

	//! @name List Items
	//@{
			SInt32						CountItems() const;
			void						MakeEmptyList();
			void						MakeList();
			void						CreateList(
												Boolean isRecord = false,
												Ptr inFactoringPtr = NULL,
												Size inFactoredSize = 0 );

			CP_AE_Descriptor			GetNthDescriptor(
												SInt32 inIndex,
												DescType inDesiredType,
												AEKeyword *inKey ) const;
	//@}

	//! @name Item Data
	//@{
			void						AddDescriptor(
												SInt32 inIndex,
												AEDesc& inData );
			void						AddDescriptor( AEDesc& inData );
			void						AddData(
												SInt32 inIndex,
												DescType inDescType,
												Ptr inData,
												Size inLength );
			void						AddData(
												DescType inDescType,
												Ptr inData,
												Size inLength );
			void						AddLong( SInt32 inLong );
			void						AddType( DescType inDescType );
	//@}
			
protected:

	// class data members
protected:
};

CPLAT_End_Namespace_CPLAT

#endif		// TARGET_OS_MAC

#endif	// _H_CP_AE_List
