//=========================================================================
//
//	FILE:				CP_Application_Imp_CE.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation class for CP_Application using Carbon
//						and Carbon Events.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 199 $
//						Latest Checkout:  $Date: 2005-06-02 14:27:57 -0700 (Thu, 02 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Application_Imp_CE
#define _H_CP_Application_Imp_CE
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_API_MAC_CARBON

	// CPLAT_Framework
#include "CP_EventHandler.h"
#include "CP_Enums.h"
#include "CP_M_CommandHandler.h"
#include "CP_M_DeferredTask.h"
#include "CP_Object.h"
#include "CP_OSStatus.h"
#include "CP_ScriptableObject.h"
#include "CP_SplashScreen.h"

CPLAT_Begin_Namespace_CPLAT

	// typedef 
typedef std::vector<CP_M_DeferredTask *, std::allocator<CP_M_DeferredTask *> >				CP_M_DeferredTask_Array;
typedef std::vector<CP_M_DeferredTask *, std::allocator<CP_M_DeferredTask *> >::iterator	CP_M_DeferredTask_Iterator;

/*!
*	@class		CP_Application_Imp
*	@brief		Carbon Events implementation of the CP_Application class.
*	
*				This object will implement the CP_Application class through
*				Carbon and Carbon Events for all of its event handling.
*/
class CP_EXPORT CP_Application_Imp : public CP_ScriptableObject, public CP_M_CommandHandler, public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Application_Imp();
	//! Destructor.
	virtual								~CP_Application_Imp() throw();
	//@}
	
	//! @name Instance
	//@{
	static CP_Application_Imp			*Instance();
	//@}

	//! @name AppleEvents
	//@{
	virtual	void						DoAE_GetData(
												const DescType inPropertyType,
												const AEDesc& inValue,
												AppleEvent& inOutReply );

	virtual	SInt32						CountContainedObjects( const DescType inType );

	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByName(
												const DescType inDesiredClass,
												const CP_String& inName );
	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByPosition(
												const DescType inDesiredClass,
												const SInt32 inPosition  );
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleQuitCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandlePreferencesCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name DeferredTasks
	//@{
			void						AddTask( CP_M_DeferredTask *inTask );
			void						RemoveTask( CP_M_DeferredTask *inTask );
			void						RemoveAllTasks();
			UInt32						GetNumberTasks();
			UInt32						GetTaskListIndex( CP_M_DeferredTask *inTask );

	virtual	void						DoDeferredTasks();			
	//@}

	//! @name Events
	//@{
	virtual	void						Run();
	//@}
			
	//! @name Exceptions
	//@{
	virtual void 						OnUncaughtException( const std::exception& inException );
	//@}

	//! @name Getters
	//@{
	virtual	CP_EventHandler&			GetEventHandler();
	//@}

	//! @name Application Status
	//@{
			CP_E_AppStatus				GetAppStatus();

										/*!
										*	Sets the status of our application from the passed in parameter.
										*	
										*	@param	inAppStatus			Flag to indicate the current condition of the application.
										*								These flags are defined in CP_Enums.h and indicate if the application
										*								is starting up, running, quitting, or quit.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetAppStatus( CP_E_AppStatus inAppStatus )				{ fAppStatus = inAppStatus; }
	//@}

	//! @name SplashScreen
	//@{
	virtual	void						ShowSplashScreen();
			CP_SplashScreen				*GetSplashScreen()										{ return( fSplashScreen ); }
	//@}

protected:
	//! @name AppleEvents
	//@{
	virtual	DescType					GetAppleClass() const;
	virtual	void						MakeSelfSpecifier(
												AEDesc& inContainerSpec,
												AEDesc& inOutSelfSpec );

	virtual	void						HandleOpenApplicationAppleEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply,
												SInt32 inRefCon );
	virtual	void						HandleQuitApplicationAppleEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply,
												SInt32 inRefCon );
	virtual	void						HandlePreferencesAppleEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply,
												SInt32 inRefCon );
	virtual	void						HandleOpenDocumentAppleEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply,
												SInt32 inRefCon );
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleNewUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandleNewCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	virtual	void						HandleOpenUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandleOpenCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	virtual	void						HandlePageSetupUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandlePageSetupCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	virtual	void						HandlePrintUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandlePrintCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }

	virtual	void						HandleCloseUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutWasHandled,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name Events
	//@{
	virtual	void						HandleOpenApplication()									{}
	virtual	void						HandleOpenDocument( CP_FileSystemSpec& /*inFileSpec*/ )	{}
	virtual	void						HandleReopenApplication()								{}
	virtual	bool						HandleQuitApplication()									{ return( false ); }
	virtual	void						HandleShowPreferences()									{}

	virtual	void						CommandProcess( CP_Event_Base& inEvent );
	//@}
	
	//! @name Documents
	//@{
	virtual	void						RegisterDocuments();
	virtual SInt32						CountDocuments();
	virtual	CP_ScriptableObject_Imp		*GetNthDocument( SInt32 inPosition );
	//@}

	//! @name Initialization
	//@{
	virtual	void						PrepareToRun();
	//@}

	//! @name Misc
	//@{
			void						QuitApplication();
	//@}

private:
	//! @name Initialization
	//@{
			void						Init();
			void						InitEventHandler( CP_EventHandler& inEventHandler );
			void						InitAppleEvents();
	//@}

	//! @name Carbon Events
	//@{
			void						AppQuit( CP_Event_Base& inEvent );
			void						CommandUpdateStatus( CP_Event_Base& inEvent );
	//@}

	//! @name Slots
	//@{
			void						HandleDeferredTasksTimer(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOSStatus );
	//@}

	// class data
protected:
	static CP_Application_Imp			*fsInstance;		/*! Single instance of this class. */
								
	CP_E_AppStatus						fAppStatus;			/*! Current state of the application, startup, running, quitting or quit. */
	CP_EventHandler						fEventHandler;		/*! Event handling object for this class. */

	CP_SplashScreen						*fSplashScreen;		/*! Instance of any splash screen displayed for this app. */

	CP_M_DeferredTask_Array				*fDeferredTasks;	/*! Lists of tasks to be handled at idle time. */
	CP_TimedTask						*fDeferredTasksTimer;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_API_MAC_CARBON

#endif	// _H_CP_Application_Imp_CE

