//=========================================================================
//
//	FILE:				CP_Event_Imp_CE.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Event class implementation for a MacOS CarbonEvent.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Event_Imp_CE
#define _H_CP_Event_Imp_CE
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_API_MAC_CARBON

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_OSStatus.h"

	// Standard Library
#include <new>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Event_Imp
*	@brief		CarbonEvents implementation of the CP_Event class.
*	
*				This class implements the features of the CP_Event class using MacOS Carbon
*				Events.
*
*				In addition to just wrapping around the sepecific event, there are many accessors
*				and setters for getting and setting the parameters of the Carbon Event.
*/
class CP_EXPORT CP_Event_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event_Imp();
										CP_Event_Imp(
												UInt32 inClassID, 
												UInt32 inKind );
										CP_Event_Imp(
												EventRef inEvent,
												EventHandlerCallRef inHandlerCallRef );
	virtual								~CP_Event_Imp();
	//@}
			
	//! @name Accessors
	//@{
										/*!
										*	operator EventRef for returning the EventRef for this event.
										*	
										*	@return	EventRef	Returns a reference to the EventRef for this event.
										*/
										operator EventRef&()						{ return fEventRef; }

										/*!
										*	Returns the EventRef for this event.
										*	
										*	@return	EventRef	Returns a reference to the EventRef for this event.
										*/
			EventRef					GetEventRef() 								{ return fEventRef; }

										/*!
										*	operator EventHanlerCallRef for returning the EventHandlerCallRef for this event.
										*	
										*	@return	EventHandlerCallRef	Returns a reference to the EventHandlerCallRef for this event.
										*/
										operator EventHandlerCallRef&()				{ return fHandlerCallRef; }

										/*!
										*	Returns the EventHandlerCallRef for this event.
										*	
										*	@return	EventHandlerCallRef	Returns a reference to the EventHandlerCallRef for this event.
										*/
			EventHandlerCallRef 		GetEventHandlerCallRef()					{ return( fHandlerCallRef ); }
	//@}

	//! @name Exceptions
	//@{
	static 	void						StoreExceptionInfo(
												EventRef inEvent, 
												const std::exception& inEx ) throw();
	static 	void						RethrowException(
												EventRef inEvent, 
												OSStatus inStatus );
	//@}

	//! @name Getters
	//@{
	virtual	UInt32						GetClass() const;
	virtual	UInt32						GetKind() const;

			OSStatus					GetParameter(
												EventParamName inName,
												EventParamType inType,
												UInt32 inBufferSize,
												void *outData );
			OSStatus					GetParameter(
												EventParamName inName,
												HICommand *outValue );
	//@}

	//! @name Retentionn
	//@{
			void						Retain();
			void						Release();
	//@}

	//! @name Sending
	//@{
			void						Send(
												EventTargetRef inTargetRef,
												OptionBits inOptionBits = 0 );
			OSStatus					Send(
												EventTargetRef inTargetRef,  
												const std::nothrow_t&,
												OptionBits inOptionBits = 0 ) throw();
	//@}

	//! @name Setters
	//@{
			void						Set(
												EventRef inEvent,
												EventHandlerCallRef inHandlerCallRef );
	//@}

	//! @name Templete Methods
	//@{
										/*!
										*	Template method for getting a parameter out of the CarbonEvent. This method
										*	is used by the specific parametter getters to return a specific type of
										*	parameter from this event.
										*	
										*	@param	inName		EventParamName of the parameter to return.
										*	@param	inType		EventParamType of the parameter to return.
										*	
										*	@return	OSStatus	CarbonEvent Manager result of the operation.
										*/
			template <class T> OSStatus	GetParameter(
												EventParamName inName,
												EventParamType inType,
												T *outValue )						{ return( GetParameter( inName, inType, sizeof( T ), outValue ) ); }
	//@}

protected:
	//! @name Copy
	//@{
			void						CopyImpFields( const CP_Event_Imp& inEventImp );
	//@}

	// class data
private:
	EventRef							fEventRef;			/*! CarbonEvent Manager EventRef for this event. */

	EventHandlerCallRef 				fHandlerCallRef;	/*! CarbonEvent Manager EventHandlerCallRef for this event. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_API_MAC_CARBON

#endif	// _H_CP_Event_Imp_CE

