//=========================================================================
//
//	FILE:				CP_SDEFParser.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A set of classes for providing AppleScript support to
//						CPLAT via a sdef scripting definition file.
//	
//	HISTORY:			1st Version - May 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev:$
//						Latest Checkout:  $Date:$
//
//=========================================================================
#ifndef _H_CP_SDEFParser
#define _H_CP_SDEFParser
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_AE_ScriptingDefines.h"
#include "CP_String.h"
#include "CP_XMLDocument.h"

	// Standard Library
#include <map>
#include <vector>

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC

typedef std::pair<AEEventClass, AEEventID>								AEEventKey;

	// forward class declarations
class CP_AEObjectSupport;

namespace CP_SDEFParsing
{

	// forward class declarations
class CP_SDEFParser;

class CP_EXPORT CP_ElementAccessor
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ElementAccessor();
										CP_ElementAccessor( const CP_ElementAccessor& inElementAccessor );
	//! Destructor.
	virtual								~CP_ElementAccessor() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the style of the element accessor.
										*	
										*	@exception	none
										*	
										*	@return		CP_String			Returns the element accessor style.
										*/
			CP_String					GetStyle() const								{ return( fStyle ); }
	//@}

	//! @name operators
	//@{
			CP_ElementAccessor&			operator = ( const CP_ElementAccessor& inElementAccessor );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the style of the element accessor.
										*
										*	@param		inStyle				The style of the element accessor.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetStyle( const CP_String& inStyle )			{ fStyle = inStyle; }
	//@}
			
protected:

	// class data
private:
	CP_String							fStyle;
};

	// typedefs
#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_ElementAccessor *, std::allocator<CP_ElementAccessor *> >				CP_ElementAccessor_Array;
	typedef std::vector<CP_ElementAccessor *, std::allocator<CP_ElementAccessor *> >::iterator		CP_ElementAccessor_Iterator;
#else
	typedef std::vector<CP_ElementAccessor *>														CP_ElementAccessor_Array;
	typedef std::vector<CP_ElementAccessor *>::iterator												CP_ElementAccessor_Iterator;
#endif

class CP_EXPORT CP_ElementDefinition
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ElementDefinition();
										CP_ElementDefinition( const CP_ElementDefinition& inElementDefinition );
	//! Destructor.
	virtual								~CP_ElementDefinition() throw();
	//@}
			
	//! @name Accessors
	//@{
			void						AddAccessor( CP_ElementAccessor  *inAccessor );
			UInt32			 			GetNumberAccessors();
			CP_ElementAccessor			*GetNthAccessor( UInt32 inIndex );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the access of the element definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String			Returns the element definition access.
										*/
			CP_String					GetAccess() const								{ return( fAccess ); }

										/*!
										*	Returns the type of the element definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String			Returns the type definition access.
										*/
			CP_String					GetType() const									{ return( fType ); }
	//@}

	//! @name operators
	//@{
			CP_ElementDefinition&		operator = ( const CP_ElementDefinition& inElementDefinition );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the access of the element definition.
										*
										*	@param		inAccess			The access of this element definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetAccess( const CP_String& inAccess )			{ fAccess = inAccess; }

										/*!
										*	Sets the type of the element definition.
										*
										*	@param		inType				The type of this element definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetType( const CP_String& inType )				{ fType = inType; }
	//@}
			
protected:

	// class data
private:
	CP_ElementAccessor_Array			fAccessors;

	CP_String							fAccess;
	CP_String							fType;
};

class CP_EXPORT CP_EventDefinition
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_EventDefinition();
										CP_EventDefinition( const CP_EventDefinition& inEventDefinition );
	//! Destructor.
	virtual								~CP_EventDefinition() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the name of this element definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The name of this element definition.
										*/
			CP_String					GetName() const									{ return( fName ); }
	//@}

	//! @name operators
	//@{
			CP_EventDefinition&			operator = ( const CP_EventDefinition& inEventDefinition );
	//@}
			
	//! @name Setters
	//@{
										/*!
										*	Sets the name of this element definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetName( const CP_String& inName )				{ fName = inName; }
	//@}

protected:

	// class data
private:
	CP_String							fName;
};

class CP_EXPORT CP_PropertyDefinition
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_PropertyDefinition();
										CP_PropertyDefinition( const CP_PropertyDefinition& inPropertyDefinition );
	//! Destructor.
	virtual								~CP_PropertyDefinition() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the access of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The access of this definition.
										*/
			CP_String					GetAccess() const								{ return( fAccess ); }

										/*!
										*	Returns the code of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		DescType		The code of this definition.
										*/
			DescType					GetCode() const									{ return( fCode ); }

										/*!
										*	Returns the name of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The name of this definition.
										*/
			CP_String					GetName() const									{ return( fName ); }

										/*!
										*	Returns the type of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The type of this definition.
										*/
			CP_String					GetType() const									{ return( fType ); }
	//@}

	//! @name operators
	//@{
			CP_PropertyDefinition&		operator = ( const CP_PropertyDefinition& inPropertyDefinition );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the access of this property definition.
										*
										*	@param		inAccess		The access of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetAccess( const CP_String& inAccess )			{ fAccess = inAccess; }

										/*!
										*	Sets the code of this property definition.
										*
										*	@param		inCode		The code of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetCode( const DescType inCode )				{ fCode = inCode; }

										/*!
										*	Sets the name of this property definition.
										*
										*	@param		inName		The name of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetName( const CP_String& inName )				{ fName = inName; }

										/*!
										*	Sets the type of this property definition.
										*
										*	@param		inType		The type of this property definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetType( const CP_String& inType )				{ fType = inType; }
	//@}
			
protected:

	// class data
private:
	CP_String							fName;
	CP_String							fType;
	DescType							fCode;
	CP_String							fAccess;
};

	// typedefs
#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_ElementDefinition *, std::allocator<CP_ElementDefinition *> >				CP_ElementDefinition_Array;
	typedef std::vector<CP_ElementDefinition *, std::allocator<CP_ElementDefinition *> >::iterator		CP_ElementDefinition_Iterator;

	typedef std::vector<CP_PropertyDefinition *, std::allocator<CP_PropertyDefinition *> >				CP_PropertyDefinition_Array;
	typedef std::vector<CP_PropertyDefinition *, std::allocator<CP_PropertyDefinition *> >::iterator	CP_PropertyDefinition_Iterator;

	typedef std::vector<CP_EventDefinition *, std::allocator<CP_EventDefinition *> >					CP_EventDefinition_Array;
	typedef std::vector<CP_EventDefinition *, std::allocator<CP_EventDefinition *> >::iterator			CP_EventDefinition_Iterator;
#else
	typedef std::vector<CP_ElementDefinition *>															CP_ElementDefinition_Array;
	typedef std::vector<CP_ElementDefinition *>::iterator												CP_ElementDefinition_Iterator;

	typedef std::vector<CP_PropertyDefinition *>														CP_PropertyDefinition_Array;
	typedef std::vector<CP_PropertyDefinition *>::iterator												CP_PropertyDefinition_Iterator;

	typedef std::vector<CP_EventDefinition *>															CP_EventDefinition_Array;
	typedef std::vector<CP_EventDefinition *>::iterator													CP_EventDefinition_Iterator;
#endif

class CP_EXPORT CP_ClassDefinition
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ClassDefinition( CP_SDEFParser *inOwnerParser );
										CP_ClassDefinition( const CP_ClassDefinition& inClassDefinition );
	//! Destructor.
	virtual								~CP_ClassDefinition() throw();
	//@}
			
	//! @name Elements
	//@{
			void						AddElementDefinition( CP_ElementDefinition  *inElementDefinition );
			UInt32			 			GetNumberElementDefinitions();
			CP_ElementDefinition 		*GetNthElementDefinition( UInt32 inIndex );
	//@}
			
	//! @name Events
	//@{
			void						AddEventDefinition( CP_EventDefinition  *inEventDefinition );
			UInt32			 			GetNumberEventDefinitions();
			CP_EventDefinition 			*GetNthEventDefinition( UInt32 inIndex );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the code of this class definition.
										*	
										*	@exception	none
										*	
										*	@return		DescType		The code of this class definition.
										*/
			DescType					GetCode() const									{ return( fCode ); }

										/*!
										*	Returns the inherits form of this class definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The inherits of this class definition.
										*/
			CP_String					GetInherits() const								{ return( fInherits ); }

										/*!
										*	Returns the name of this class definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The name of this class definition.
										*/
			CP_String					GetName() const									{ return( fName ); }

										/*!
										*	Returns the plural form of this class definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The plural form of this class definition.
										*/
			CP_String					GetPlural() const								{ return( fPlural ); }
	//@}

	//! @name operators
	//@{
			CP_ClassDefinition&			operator = ( const CP_ClassDefinition& inClassDefinition );
	//@}

	//! @name Properties
	//@{
			void						AddPropertyDefinition( CP_PropertyDefinition  *inPropertyDefinition );
			UInt32			 			GetNumberPropertyDefinitions();
			CP_PropertyDefinition 		*GetNthPropertyDefinition( UInt32 inIndex );

			bool						IsClassPropertyDefined( DescType inPropertyID );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the code of this class definition.
										*
										*	@param		inCode		The code for this class definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetCode( const DescType& inCode )				{ fCode = inCode; }

										/*!
										*	Sets the inherited version of this class definition.
										*
										*	@param		inCode		The inherited version for this class definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetInherits( const CP_String& inInherits )		{ fInherits = inInherits; }

										/*!
										*	Sets the name of this class definition.
										*
										*	@param		inname		The name for this class definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetName( const CP_String& inName )				{ fName = inName; }

										/*!
										*	Sets the plural version of this class definition.
										*
										*	@param		inPlural	The plural version for this class definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetPlural( const CP_String& inPlural )			{ fPlural = inPlural; }
	//@}
			
protected:

private:
	//! @name Properties
	//@{
			void						AddPropertyListEnding();
	//@}

	// class data
private:
	CP_SDEFParser 						*fOwnerParser;
	
	CP_String							fName;
	CP_String							fPlural;
	DescType							fCode;
	CP_String							fInherits;

	CP_ElementDefinition_Array			fElementDefinitions;
	CP_PropertyDefinition_Array			fPropertyDefinitions;
	CP_EventDefinition_Array			fEventDefinitions;
};

class CP_EXPORT CP_CommandParameterDefinition
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_CommandParameterDefinition();
										CP_CommandParameterDefinition( const CP_CommandParameterDefinition& inCommandParameterDefinition );
	//! Destructor.
	virtual								~CP_CommandParameterDefinition() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the name of this command parameter definition.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The name of this command parameter definition.
										*/
		CP_String						GetName() const									{ return( fName ); }
	//@}

	//! @name operators
	//@{
		CP_CommandParameterDefinition&	operator = ( const CP_CommandParameterDefinition& inCommandParameterDefinition );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the name of this class definition.
										*
										*	@param		inName		The name for this class definition.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetName( const CP_String& inName )				{ fName = inName; }
	//@}
			
protected:

	// class data
private:
	CP_String							fName;
};

#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_CommandParameterDefinition *, std::allocator<CP_CommandParameterDefinition *> >				CP_CommandParametersDefinition_Array;
	typedef std::vector<CP_CommandParameterDefinition *, std::allocator<CP_CommandParameterDefinition *> >::iterator	CP_CommandParameterDefinition_Iterator;
#else
	typedef std::vector<CP_CommandParameterDefinition *>																CP_CommandParameterDefinition_Array;
	typedef std::vector<CP_CommandParameterDefinition *>::iterator														CP_CommandParameterDefinition_Iterator;
#endif

class CP_EXPORT CP_CommandDefinition
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_CommandDefinition();
										CP_CommandDefinition( const CP_CommandDefinition& inCommandDefinition );
	//! Destructor.
	virtual								~CP_CommandDefinition() throw();
	//@}
			
	//! @name Getters
	//@{
			CP_String					GetCode() const									{ return( fCode ); }
			CP_String					GetHidden() const								{ return( fHidden ); }
			CP_String					GetName() const									{ return( fName ); }
			AEEventKey					GetEventKey();
	//@}

	//! @name operators
	//@{
			CP_CommandDefinition&		operator = ( const CP_CommandDefinition& inCommandDefinition );
	//@}

	//! @name Parameters
	//@{
			void							AddCommandParameterDefinition( CP_CommandParameterDefinition *inCommandParameterDefinition );
			UInt32			 				GetNumberCommandParameterDefinitions();
			CP_CommandParameterDefinition 	*GetNthCommandParameterDefinition( UInt32 inIndex );
	//@}

	//! @name Setters
	//@{
			void						SetCode( const CP_String& inCode )				{ fCode = inCode; }
			void						SetHidden( const CP_String& inHidden )			{ fHidden = inHidden; }
			void						SetName( const CP_String& inName )				{ fName = inName; }
	//@}
			
protected:

	// class data
private:
	CP_CommandParameterDefinition_Array	fCommandParameters;
	
	CP_String							fCode;
	CP_String							fHidden;
	CP_String							fName;
};

	// typedefs
#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_ClassDefinition *, std::allocator<CP_ClassDefinition *> >				CP_ClassDefinition_Array;
	typedef std::vector<CP_ClassDefinition *, std::allocator<CP_ClassDefinition *> >::iterator		CP_ClassDefinition_Iterator;

	typedef std::map<DescType, CP_SDEFParsing::CP_ClassDefinition>									CP_ClassDefinition_Map;
	typedef std::map<DescType, CP_SDEFParsing::CP_ClassDefinition>::iterator						CP_ClassDefinition_MapIterator;
	typedef CP_ClassDefinition_Map::value_type														CP_ClassDefinition_MapValueType;

	typedef std::vector<CP_CommandDefinition *, std::allocator<CP_CommandDefinition *> >			CP_CommandDefinition_Array;
	typedef std::vector<CP_CommandDefinition *, std::allocator<CP_CommandDefinition *> >::iterator	CP_CommandDefinition_Iterator;
#else
	typedef std::vector<CP_ClassDefinition *>														CP_ClassDefinition_Array;
	typedef std::vector<CP_ClassDefinition *>::iterator												CP_ClassDefinition_Iterator;

	typedef std::map<DescType, CP_SDEFParsing::CP_ClassDefinition>									CP_ClassDefinition_Map;
	typedef std::map<DescType, CP_SDEFParsing::CP_ClassDefinition>::iterator						CP_ClassDefinition_MapIterator;
	typedef CP_ClassDefinition_Map::value_type														CP_ClassDefinition_MapValueType;

	typedef std::vector<CP_CommandDefinition *>														CP_CommandDefinition_Array;
	typedef std::vector<CP_CommandDefinition *>::iterator											CP_CommandDefinition_Iterator;
#endif

class CP_EXPORT CP_SDEFParser
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SDEFParser();
	//! Destructor.
	virtual								~CP_SDEFParser() throw();
	//@}
			
	//! @name Parsing
	//@{
			void						ParseSDEF( SInt16 inSDEFXMLResID );
			void						ParseSDEF( CP_CFURL& inSdefUrl );
	//@}
			
	//! @name Classes
	//@{
			void						AddClassDefinition( CP_ClassDefinition *inClassDefinition );
			UInt32			 			GetNumberClassDefinitions();
			CP_ClassDefinition 			*GetNthClassDefinition( UInt32 inIndex );
			CP_ClassDefinition_Array	GetClassDefinitions()													{ return( fClassDefinitions ); }

			bool						IsClassPropertyDefined(
												DescType inClassID,
												DescType inPropertyID );
	//@}

	//! @name Commands
	//@{
			void						AddCommandDefinition( CP_CommandDefinition *inCommandDefinition );
			UInt32			 			GetNumberCommandDefinitions();
			CP_CommandDefinition 		*GetNthCommandDefinition( UInt32 inIndex );
			CP_CommandDefinition_Array	GetCommandDefinitions()													{ return( fCommandDefinitions ); }
			CP_CommandDefinition 		*FindCommandByName( const CP_String& inCommandName );
	//@}
			
protected:
			
private:
	//! @name Parsing
	//@{
			void						ParseNodes( CP_XMLDocNode *inNode );

			void						ParseClasses( CP_XMLDocNode *inNode );
			void						ParseClass( CP_XMLDocNode *inNode );
			void						ParseClassDefinition( CP_XMLDocNode *inNode );

			void						ParseClassElements(
												CP_XMLDocNode *inNode,
												CP_ClassDefinition *inClassDefinition );
			void						ParseClassElement(
												CP_XMLDocNode *inNode,
												CP_ClassDefinition *inClassDefinition );
			void						ParseElementAccessor(
												CP_XMLDocNode *inNode,
												CP_ElementDefinition *inElementDefinition );

			void						ParseClassProperties(
												CP_XMLDocNode *inNode,
												CP_ClassDefinition *inClassDefinition );
			void						ParseClassProperty(
												CP_XMLDocNode *inNode,
												CP_ClassDefinition *inClassDefinition );

			void						ParseClassEvents(
												CP_XMLDocNode *inNode,
												CP_ClassDefinition *inClassDefinition );
			void						ParseClassEvent(
												CP_XMLDocNode *inNode,
												CP_ClassDefinition *inClassDefinition );

			void						ParseCommands( CP_XMLDocNode *inNode );
			void						ParseCommand( CP_XMLDocNode *inNode );
	//@}

	// class data
private:
	CP_ClassDefinition_Array			fClassDefinitions;
	CP_CommandDefinition_Array			fCommandDefinitions;
};
	
}	// namespace CP_SDEFParsing

#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SDEFParser

