//=========================================================================
//
//	FILE:				CP_BackgroundView.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to draw a view in a solid color for use
//						as a background view.
//	
//	HISTORY:			1st Version - February 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev:$
//						Latest Checkout:  $Date:$
//
//=========================================================================
#ifndef _H_CP_BackgroundView
#define _H_CP_BackgroundView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_ARGBColor.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_BackgroundView
*	@brief		CP_View class to draw a view in a solid color for use as a background view.
*/
class CP_BackgroundView : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_BackgroundView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_BackgroundView() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the color used in filling the background.
										*	
										*	@exception	none
										*	
										*	@return		CP_ARGBColor		The color to use.
										*/
			CP_ARGBColor				GetColor() const								{ return( fColor ); }
	//@}

	//! @name Setters
	//@{
			void						SetColor( const CP_ARGBColor& inColor );
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}
			
	// class data
protected:
	CP_ARGBColor						fColor;		/*! Color used in filling the background. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Drawing
	//@{
	virtual	void						SetForeBackColors();
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_BackgroundView

