//=========================================================================
//
//	FILE:				CP_DisclosureTriangle.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms disclosure triangle control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_DisclosureTriangle
#define _H_CP_DisclosureTriangle
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_DisclosureTriangle
*	@brief		Implementation of a platforms disclosure triangle control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_DisclosureTriangle : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DisclosureTriangle();
										CP_DisclosureTriangle(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												UInt16 inOrientation = 0,
												bool inAutoToggle = true );
	//! Destructor.
	virtual								~CP_DisclosureTriangle() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:

	#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		CP_String						fText;
		bool							fIsPushed;
	#endif
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												UInt16 inOrientation = 0,
												bool inAutoToggle = true );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Slots
	//@{
	virtual	void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

	//! @name Events
	//@{
	virtual	void						HandleMouseDown( CP_Event_Base& inEvent );
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												UInt16 inOrientation = 0,
												bool inAutoToggle = true,
												bool inDrawTitle = false );
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Events
	//@{
	virtual	void						HandleMouseDown( CP_Event_Base& inEvent );
	//@}

	//! @name Slots
	//@{
	virtual	void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												UInt16 inOrientation = 0,
												bool inAutoToggle = true,
												bool inDrawTitle = false );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DisclosureTriangle
