//=========================================================================
//
//	FILE:				CP_M_SubViewHandler.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for UI elements that can contain sub views.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 223 $
//						Latest Checkout:  $Date: 2005-06-28 23:07:49 -0700 (Tue, 28 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_M_SubViewHandler
#define _H_CP_M_SubViewHandler
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Point.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_View;
class CP_Window;

	// typedefs
/*!	\var typedef CP_SubViews_List
 *	std::vector that contains the list of subviews for this view. 
 */
/*!	\var typedef CP_SubViews_Iterator
 *	forward iterator to iterator over the CP_SubViews_List. 
 */
/*!	\var typedef CP_SubViews_ReverseIterator
 *	reverse iterator to iterator over the CP_SubViews_List. 
 */
#if TARGET_OS_LINUX
	typedef std::vector<CP_View * >													CP_SubViews_List;
	typedef std::vector<CP_View * >::iterator										CP_SubViews_Iterator;
	typedef std::vector<CP_View * >::reverse_iterator								CP_SubViews_ReverseIterator;
#else
	typedef std::vector<CP_View *, std::allocator<CP_View *> >						CP_SubViews_List;
	typedef std::vector<CP_View *, std::allocator<CP_View *> >::iterator			CP_SubViews_Iterator;
	typedef std::vector<CP_View *, std::allocator<CP_View *> >::reverse_iterator	CP_SubViews_ReverseIterator;
#endif

/*!
*	@class		CP_M_SubViewHandler
*	@brief		Mix-in class for UI elements that can contain sub views.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_M_SubViewHandler
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_M_SubViewHandler();
	//@}
			
	//! @name Subviews
	//@{
			void						AddSubviewToList( CP_View *inSubView );
	virtual	void						DeleteSubViews();
	virtual	void						DeleteSubView(
												CP_View *inSubView,
												bool inDeleteSubViews = true,
												bool inDeleteSubView = true );
			void						RemoveSubView( CP_View *inSubView );

			CP_View						*FindSubView(
												const CP_Point& inWhere,
												bool inRecrusive = kCP_NoRecrusiveSearch );
			CP_View						*FindSubView(
												const UInt32 inViewID,
												bool inRecrusive = kCP_NoRecrusiveSearch );
			bool						ContainsSubView(
												CP_View *inView,
												bool inRecrusive );
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_SubViewHandler();
	//@}

	//! @name Appearance
	//@{
	virtual	void						ShowSubs();
	virtual	void						HideSubs();
	//@}

	// class data
protected:
	CP_SubViews_List					*fSubViews;		/*! List of subviews contained in this view. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_SubViewHandler

