//=========================================================================
//
//	FILE:				CP_PushButton.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms push button control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_PushButton
#define _H_CP_PushButton
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_PushButton
*	@brief		Implementation of a platforms push button control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_PushButton : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_PushButton();
								CP_PushButton(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										const CP_String& inTitle,
										UInt32 inCommand,
										bool inIsDefault = false );
	//! Destructor.
	virtual						~CP_PushButton() throw();
	//@}
			
	//! @name Default / Cancel
	//@{
			void				SetCancel(
										const CP_String& inTitle,
										bool inIsDefault );
			void				SetDefault(
										const CP_String& inTitle,
										bool inIsDefault );
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist			*PersistCreator();
	//@}
			
protected:
	//! @name Events
	//@{
	virtual	void				InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void				OnInitPersist(
										CP_View *inSuperview,
										CP_DataValueList& inDataValueList );
	//@}

	//! @name Slots
	//@{
			void				ControlClicked( CP_DataValueHolder& inValue );
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										const CP_String& inTitle,
										UInt32 inCommand );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										const CP_String& inTitle,
										UInt32 inCommand );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										const CP_String& inTitle,
										UInt32 inCommand );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PushButton
