//=========================================================================
//
//	FILE:				CP_Slider.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms push button control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Slider
#define _H_CP_Slider
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Slider
*	@brief		Implementation of a platforms push button control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Slider : public CP_PlatformControl
{
public:
	enum {
	  kCP_SliderPointsDownOrRight = 0,
	  kCP_SliderPointsUpOrLeft  = 1,
	  kCP_SliderDoesNotPoint    = 2
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Slider();
										CP_Slider(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												SInt32 inOrientation = 0,
												UInt16 inNumberTicks = 0,
												bool inLiveTracking = true );
	//! Destructor.
	virtual								~CP_Slider() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Value
	//@{
	virtual	SInt32						GetValue();
	virtual	void						SetValue( SInt32 inValue );
			
	virtual	SInt32						GetMin();
	virtual	void						SetMin( SInt32 inValue );
	
	virtual	SInt32						GetMax();
	virtual	void						SetMax( SInt32 inValue );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	
			void						HandleButtonPushedEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	//! @name Slots
	//@{
			void						ControlClicked( CP_DataValueHolder& inValue );
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
	//! @name Scrolling
	//@{
	virtual	void						DoSliderAction_Mac( SInt16 inPart );
	//@}

private:
	//! @name Scrolling
	//@{
	static pascal void					SliderAction(
												ControlHandle inPlatformControl,
												SInt16 inPart );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												SInt32 inOrientation = 0,
												UInt16 inNumberTicks = 0,
												bool inLiveTracking = true );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Events
	//@{
			void						HandleHScrollEvent( CP_Event_Base& inEvent );
			void						HandleVScrollEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												SInt32 inOrientation = 0,
												UInt16 inNumberTicks = 0,
												bool inLiveTracking = true );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												SInt32 inOrientation = 0,
												UInt16 inNumberTicks = 0,
												bool inLiveTracking = true );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Slider
