//=========================================================================
//
//	FILE:				CP_TableGeometry_Mono.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_TableGeometry_Base class for maining single value
//						geometry for a table. Single value geometry means that
//						all rows are the same height, and all cells in a column
//						are the same width.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 205 $
//						Latest Checkout:  $Date: 2005-06-07 20:17:23 -0700 (Tue, 07 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_TableGeometry_Mono
#define _H_CP_TableGeometry_Mono
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_TableGeometry_Base.h"

CPLAT_Begin_Namespace_CPLAT

	// forward declarations
class CP_TableView;

/*!
*	\class		CP_TableGeometry_Mono
*	\brief		CP_TableGeometry_Base class for maining single value geometry for a table.
*				Single value geometry means that all rows are the same height, and all cells
*				in a column are the same width.
*/
class CP_EXPORT CP_TableGeometry_Mono : public CP_TableGeometry_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TableGeometry_Mono( CP_TableView *inTableView );
	//! Destructor.
	virtual								~CP_TableGeometry_Mono() throw();
	//@}
		
	//! @name Columns
	//@{
	virtual SInt32						GetNumberColumns()									{ return( fNumberColumns ); }
	virtual	SInt32						GetNumberColumns( SInt32 /*inRow*/ )				{ return( fNumberColumns ); }
	virtual void						SetNumberColumns(
												SInt32 /*inRow*/,
												SInt32 inNumberColumns )					{ fNumberColumns = inNumberColumns; }

	virtual SInt32						GetColumnWidth(
												SInt32 inRow,
												SInt32 inColumn );
	virtual void						SetColumnWidth(
												SInt32 /*inRow*/,
												SInt32 inColumn,
												SInt32 inColumnWidth )						{ fColumnWidth = inColumnWidth; SendColumnWidthChangedSignal( inColumn ); }
	
	virtual	CP_Rect						GetColumnBounds(
												const CP_Rect& inRowBounds,
												SInt32 inRow,
												SInt32 inColumn );
	virtual	CP_Rect						GetColumnBounds(
												SInt32 inRow,
												SInt32 inColumn );
	//@}

	//! @name Rows
	//@{
	virtual SInt32						GetNumberRows()										{ return( fNumberRows ); }
	virtual void						SetNumberRows( SInt32 inNumberRows );

	virtual SInt32						GetRowHeight( SInt32 /*inRow*/ )					{ return( fRowHeight ); }
	virtual void						SetRowHeight(
												SInt32 inRow,
												SInt32 inRowHeight )						{ fRowHeight = inRowHeight; SendRowHeightChangedSignal( inRow ); }

	virtual	CP_Point					GetRowLocation( SInt32 inRow );

	virtual	void						AddRow()											{ fNumberRows++; SendRowAddedSignal(); }
	//@}

	//! @name Testers
	//@{
	virtual	SInt32						GetRowHitBy( const CP_Point& inHitPoint );
	virtual	SInt32						GetColumnHitBy( const CP_Point& inHitPoint );
	//@}	

protected:

	// class data
protected:
	SInt32								fNumberRows;
	SInt32								fNumberColumns;
	
	SInt32								fRowHeight;
	SInt32								fColumnWidth;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TableGeometry_Mono

