//=========================================================================
//
//	FILE:				CP_TableGeometry_Multi.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_TableGeometry_Base class for maining multi value
//						geometry for a table. Multi value geometry means that
//						rows and columns can be different sizes.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 110 $
//						Latest Checkout:  $Date: 2005-03-23 14:37:56 -0800 (Wed, 23 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_TableGeometry_Multi
#define _H_CP_TableGeometry_Multi
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_TableGeometry_Base.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward declarations
class CP_TableView;

#if TARGET_OS_LINUX
	typedef std::vector<SInt32, std::allocator<SInt32> >								CP_Dimension_Array;
	typedef std::vector<SInt32, std::allocator<SInt32> >::iterator						CP_Dimension_Iterator;
	typedef std::vector<SInt32, std::allocator<SInt32> >::reverse_iterator				CP_Dimension_ReverseIterator;
#else
	typedef std::vector<SInt32>															CP_Dimension_Array;
	typedef std::vector<SInt32>::iterator												CP_Dimension_Iterator;
	typedef std::vector<SInt32>::reverse_iterator										CP_Dimension_ReverseIterator;
#endif

/*!
*	\class		CP_TableGeometry_Multi
*	\brief		CP_TableGeometry_Base class for maining multi value geometry for a table.
*				Multi value geometry means that rows and columns can be different sizes.
*/
class CP_EXPORT CP_TableGeometry_Multi : public CP_TableGeometry_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TableGeometry_Multi( CP_TableView *inTableView );
	//! Destructor.
	virtual								~CP_TableGeometry_Multi() throw();
	//@}
		
	//! @name Columns
	//@{
	virtual	SInt32						GetNumberColumns( SInt32 inRow );
	virtual void						SetNumberColumns(
												SInt32 inRow,
												SInt32 inNumberColumns );

	virtual SInt32						GetColumnWidth(
												SInt32 inRow,
												SInt32 inColumn );
	virtual void						SetColumnWidth(
												SInt32 inRow,
												SInt32 inColumn,
												SInt32 inColumnWidth );

	virtual	CP_Rect						GetColumnBounds(
												const CP_Rect& inRowBounds,
												SInt32 inRow,
												SInt32 inColumn );
	virtual	CP_Rect						GetColumnBounds(
												SInt32 inRow,
												SInt32 inColumn );
	//@}

	//! @name Rows
	//@{
	virtual SInt32						GetNumberRows();
	virtual void						SetNumberRows( SInt32 inNumberRows );

	virtual SInt32						GetRowHeight( SInt32 inRow );
	virtual void						SetRowHeight(
												SInt32 inRow,
												SInt32 inRowHeight );

	virtual	CP_Point					GetRowLocation( SInt32 inRow );

			void						AddRow();
	//@}

	//! @name Testers
	//@{
	virtual	SInt32						GetRowHitBy( const CP_Point& inHitPoint );
	virtual	SInt32						GetColumnHitBy( const CP_Point& inHitPoint );
	//@}

protected:

	// class data
protected:
	CP_Dimension_Array					fRowHeights;
	CP_Dimension_Array					fColWidths;
	SInt32								fDefaultRowHeight;
	SInt32								fDefaultColWidth;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TableGeometry_Multi

