//=========================================================================
//
//	FILE:				CP_TextGroupBox.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms text group box control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_TextGroupBox
#define _H_CP_TextGroupBox
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_TextGroupBox
*	@brief		Implementation of a platforms text group box control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_TextGroupBox : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TextGroupBox();
										CP_TextGroupBox(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												CP_E_GroupBoxStyle inGroupBoxStyle = kCP_GroupBox_Primary );
	//! Destructor.
	virtual								~CP_TextGroupBox() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:

	#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		CP_String						fText;				/*! Text displayed for the title of the popup. */
		CP_E_GroupBoxStyle 				fGroupBoxStyle;		/*! Type of group box - primary or secondary. */
	#endif
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												CP_E_GroupBoxStyle inGroupBoxStyle = kCP_GroupBox_Primary );
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												CP_E_GroupBoxStyle inGroupBoxStyle = kCP_GroupBox_Primary );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												CP_E_GroupBoxStyle inGroupBoxStyle = kCP_GroupBox_Primary );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TextGroupBox
